/*
 *   Copyright 2017-2020, Schlumberger
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 */

package org.opengroup.osdu.notification.api;

import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.opengroup.osdu.core.common.http.HttpResponse;
import org.opengroup.osdu.core.common.logging.JaxRsDpsLog;
import org.opengroup.osdu.notification.provider.interfaces.IPubsubRequestBodyExtractor;
import org.opengroup.osdu.notification.service.NotificationHandler;
import org.powermock.modules.junit4.PowerMockRunner;
import org.springframework.http.ResponseEntity;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.ArgumentMatchers.anyString;
import static org.mockito.Mockito.when;

@RunWith(PowerMockRunner.class)
public class PubsubEndpointTests {
    @Mock
    private IPubsubRequestBodyExtractor pubsubRequestBodyExtractor;
    @Mock
    private NotificationHandler notificationHandler;
    @Mock
    private JaxRsDpsLog log;
    @InjectMocks
    private PubsubEndpoint sut;

    private static final String NOTIFICATION_ID = "test-notification-id";
    private static final String PUBSUB_MESSAGE = "test-pubsub-message-data";
    private HttpResponse response = new HttpResponse();

    @Before
    public void setup() throws Exception {
        when(this.pubsubRequestBodyExtractor.extractNotificationIdFromRequestBody()).thenReturn(NOTIFICATION_ID);
        when(this.pubsubRequestBodyExtractor.extractDataFromRequestBody()).thenReturn(PUBSUB_MESSAGE);
    }

    @Test
    public void should_return200_whenPubsubMessageValidAndSuccessCodeReturnedFromNotificationHandler() throws Exception {
        response.setResponseCode(200);
        when(this.notificationHandler.NotifySubscriber(anyString(), anyString(), any())).thenReturn(response);
        ResponseEntity responseEntity = this.sut.recordChanged();
        Assert.assertEquals(200, responseEntity.getStatusCode().value());
        Assert.assertEquals("message acknowledged by client", responseEntity.getBody().toString());
    }

    @Test
    public void should_return400_whenPubsubMessageValidAndFailureCodeReturnedFromNotificationHandler() throws Exception {
        response.setResponseCode(400);
        when(this.notificationHandler.NotifySubscriber(anyString(), anyString(), any())).thenReturn(response);
        ResponseEntity responseEntity = this.sut.recordChanged();
        Assert.assertEquals(400, responseEntity.getStatusCode().value());
        Assert.assertEquals("message not acknowledged by client", responseEntity.getBody().toString());
    }

    @Test
    public void should_return400_whenPubsubMessageValidAndNotificationHandlerThrowsException() throws Exception {
        response.setResponseCode(400);
        when(this.notificationHandler.NotifySubscriber(anyString(), anyString(), any())).thenThrow(new Exception("error"));
        ResponseEntity responseEntity = this.sut.recordChanged();
        Assert.assertEquals(400, responseEntity.getStatusCode().value());
        Assert.assertEquals("message not acknowledged by client", responseEntity.getBody().toString());
    }
}
