package org.opengroup.osdu.notification.service;

import com.fasterxml.jackson.databind.ObjectMapper;
import org.junit.Assert;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.opengroup.osdu.core.common.http.HttpResponse;
import org.opengroup.osdu.core.common.logging.JaxRsDpsLog;
import org.opengroup.osdu.core.common.model.http.AppException;
import org.opengroup.osdu.core.common.model.http.DpsHeaders;
import org.opengroup.osdu.core.common.model.notification.Subscription;
import org.opengroup.osdu.core.common.notification.ISubscriptionFactory;
import org.opengroup.osdu.core.common.notification.ISubscriptionService;
import org.opengroup.osdu.core.common.notification.SubscriptionException;
import org.opengroup.osdu.core.common.notification.SubscriptionService;
import org.opengroup.osdu.notification.di.SubscriptionCacheFactory;
import org.powermock.modules.junit4.PowerMockRunner;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import static org.junit.Assert.fail;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.ArgumentMatchers.anyString;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

@RunWith(PowerMockRunner.class)
public class SubscriptionHandlerTests {
    @Mock
    private ISubscriptionFactory subscriptionFactory;
    @Mock
    private SubscriptionCacheFactory subscriptionCacheFactory;
    @Mock
    private JaxRsDpsLog log;
    @Mock
    private ObjectMapper objectMapper;
    @InjectMocks
    private SubscriptionHandler sut;

    private static final String NOTIFICATION_ID = "test-notification-id";

    @Test(expected = AppException.class)
    public void should_throwException_whenSubscriptionNotFound() throws Exception {
        when(this.subscriptionCacheFactory.get(any())).thenReturn(null);
        ISubscriptionService subscriptionService = mock(SubscriptionService.class);
        when(this.subscriptionFactory.create(any())).thenReturn(subscriptionService);
        HttpResponse response = new HttpResponse();
        when(subscriptionService.query(any())).thenThrow(new SubscriptionException("error", response));
        this.sut.getSubscriptionFromCache(this.NOTIFICATION_ID);
        fail("should throw AppException");
    }

    @Test(expected = AppException.class)
    public void should_throwException_whenSubscriptionParsingErrorOccurs() throws Exception {
        when(this.subscriptionCacheFactory.get(any())).thenReturn(null);
        ISubscriptionService subscriptionService = mock(SubscriptionService.class);
        when(this.subscriptionFactory.create(any())).thenReturn(subscriptionService);
        String jsonSubscription = this.getHmacSubscription();
        ObjectMapper objectMapper = new ObjectMapper();
        Subscription subscription = objectMapper.readValue(jsonSubscription, Subscription.class);
        List<Subscription> queryResult = new ArrayList<>();
        queryResult.add(subscription);
        when(subscriptionService.query(any())).thenReturn(queryResult);
        sut.setObjectMapper(this.objectMapper);
        when(this.objectMapper.readValue(anyString(), any(Class.class))).thenThrow(new IOException());
        this.sut.getSubscriptionFromCache(NOTIFICATION_ID);
        fail("should throw AppException");
    }

    @Test
    public void should_return200_whenSubscriptionGotFromRegistration() throws Exception {
        when(this.subscriptionCacheFactory.get(any())).thenReturn(null);
        ISubscriptionService subscriptionService = mock(SubscriptionService.class);
        when(this.subscriptionFactory.create(any())).thenReturn(subscriptionService);
        sut.setObjectMapper(new ObjectMapper());
        String jsonSubscription = this.getHmacSubscription();
        ObjectMapper objectMapper = new ObjectMapper();
        Subscription subscription = objectMapper.readValue(jsonSubscription, Subscription.class);
        List<Subscription> queryResult = new ArrayList<>();
        queryResult.add(subscription);
        when(subscriptionService.query(any())).thenReturn(queryResult);
        Subscription response = this.sut.getSubscriptionFromCache(NOTIFICATION_ID);
        Assert.assertEquals("testSubscription",response.getName());
    }

    private String getHmacSubscription() {
        return "{\n" +
                "\t\"name\": \"testSubscription\",\n" +
                "\t\"description\": \"Description\",\n" +
                "\t\"topic\": \"records-changed\",\n" +
                "\t\"pushEndpoint\": \"http://challenge\",\n" +
                "\t\"notificationId\": \"notificationId\",\n" +
                "\t\"id\": \"id_1\",\n" +
                "\t\"createdBy\": \"test@test.com\",\n" +
                "\t\"secret\": {\n" +
                "\t\t\"secretType\": \"HMAC\",\n" +
                "\t\t\"value\": \"testsecret\"\n" +
                "\t}\n" +
                "}";
    }
}
